/***************************************************************************
 *   Name:      Music_Format.h                                             *
 *   Purpose:   Defines Music Format support                               *
 *   Copyright (C) 2008 by Jackie Team                                     *
 *   jackie.team@gmail.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MUSIC_FORMAT_H
#define MUSIC_FORMAT_H
#include <taglib/fileref.h>
#include <taglib/tag.h>
#include <taglib/id3v1tag.h>
#include <taglib/id3v2tag.h>
#include <taglib/mpegfile.h>
#include <wx/filename.h>
#ifdef USE_MP4
#include <taglib/mp4file.h>
#endif
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

/**
 * Abstract class MusicFormat
 * Allows to create a derived class for each format
 */

class MusicFormat {
protected:
    TagLib::File* file;
public:
    static wxString formatToTime(int);

    MusicFormat() {}
    MusicFormat(wxString f) {}
    virtual ~MusicFormat() {}
    virtual bool exists() {
        return false;
    }
    virtual wxString getArtist() {
        return wxEmptyString;
    }
    virtual wxString getTitle() {
        return wxEmptyString;
    }
    virtual wxString getAlbum() {
        return wxEmptyString;
    }
    virtual wxString getGenre() {
        return wxEmptyString;
    }
    virtual wxString getTrack() {
        return wxEmptyString;
    }
    virtual wxString getYear() {
        return wxEmptyString;
    }
    virtual wxString getLength() {
        return _("?");
    }
    virtual void setArtist(wxString) {}
    virtual void setTitle(wxString) {}
    virtual void setAlbum(wxString) {}
    virtual void setGenre(wxString) {}
    virtual void setTrack(int) {}
    virtual void setYear(int) {}
    virtual void save() {}
};

class MusicFormatID3V1:public MusicFormat {
protected:
    TagLib::MPEG::File* file;
public:
    MusicFormatID3V1(wxString f);
    virtual ~MusicFormatID3V1() {
        delete file;
    }
    virtual bool exists();
    virtual wxString getArtist();
    virtual wxString getTitle();
    virtual wxString getAlbum();
    virtual wxString getGenre();
    virtual wxString getTrack();
    virtual wxString getYear();
    virtual wxString getLength();
    virtual void setArtist(wxString);
    virtual void setTitle(wxString);
    virtual void setAlbum(wxString);
    virtual void setGenre(wxString);
    virtual void setTrack(int);
    virtual void setYear(int);
    virtual void save();
};


class MusicFormatID3V2:public MusicFormat {
protected:
    TagLib::MPEG::File* file;
public:
    MusicFormatID3V2(wxString f);
    virtual ~MusicFormatID3V2() {
        delete file;
    }
    virtual bool exists();
    virtual wxString getArtist();
    virtual wxString getTitle();
    virtual wxString getAlbum();
    virtual wxString getGenre();
    virtual wxString getTrack();
    virtual wxString getYear();
    virtual wxString getLength();
    virtual void setArtist(wxString);
    virtual void setTitle(wxString);
    virtual void setAlbum(wxString);
    virtual void setGenre(wxString);
    virtual void setTrack(int);
    virtual void setYear(int);
    virtual void save();
};

class MusicFormatOGG:public MusicFormat {
protected:
    TagLib::FileRef* file;
public:
    MusicFormatOGG() {}
    MusicFormatOGG(wxString f);
    virtual ~MusicFormatOGG() {
        delete file;
    }
    virtual bool exists();
    virtual wxString getArtist();
    virtual wxString getTitle();
    virtual wxString getAlbum();
    virtual wxString getGenre();
    virtual wxString getTrack();
    virtual wxString getYear();
    virtual wxString getLength();
    virtual void setArtist(wxString);
    virtual void setTitle(wxString);
    virtual void setAlbum(wxString);
    virtual void setGenre(wxString);
    virtual void setTrack(int);
    virtual void setYear(int);
    virtual void save();
};

class MusicFormatFLAC:public MusicFormatOGG {
public:
    MusicFormatFLAC(wxString f);
};

#ifdef USE_MP4
class MusicFormatMP4:public MusicFormat {
protected:
    TagLib::MP4::File* file;
public:
    MusicFormatMP4(wxString f);
    virtual ~MusicFormatMP4() {
        delete file;
    }
    virtual bool exists();
    virtual wxString getArtist();
    virtual wxString getTitle();
    virtual wxString getAlbum();
    virtual wxString getGenre();
    virtual wxString getTrack();
    virtual wxString getYear();
    virtual wxString getLength();
    virtual void setArtist(wxString);
    virtual void setTitle(wxString);
    virtual void setAlbum(wxString);
    virtual void setGenre(wxString);
    virtual void setTrack(int);
    virtual void setYear(int);
    virtual void save();
};
#endif

#endif
