/***************************************************************************
 * Name:      ScanThread.cpp                                               *
 * Purpose:   Code for the thread wich scan directory                      *
 * Author:    Jackie Team (jackie.team@gmail.com)                          *
 * Created:   2007-12-16                                                   *
 * Copyright: Jackie Team (www.musicorganizer.org)                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef WX_PRECOMP
#include "wx_pb.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "components.h"

wxDirTraverserMp3::wxDirTraverserMp3(wxArrayString* files,bool recursive)  {
    m_files=files;
    rec= recursive;
    stop = false;

}

/* ****************************************************************************************** */

wxDirTraverseResult wxDirTraverserMp3::OnFile(const wxString& filename) {
    wxFileName fn(filename);
    wxString ext=fn.GetExt().MakeLower();
#ifdef USE_MP4
    if(ext==_T("mp3") || ext==_T("flac") || ext==_T("ogg") || ext==_T("m4a") || ext==_T("aac") || ext==_T("m4p"))//An mp3/flac/ogg/m4a file has been found
#else
    if(ext==_T("mp3") || ext==_T("flac") || ext==_T("ogg"))//An mp3/flac/ogg file has been found
#endif
        m_files->Add(filename);
    if(stop)//if the thread is beein aborting
        return wxDIR_STOP;
    else
        return wxDIR_CONTINUE;
}

/* ****************************************************************************************** */

wxDirTraverseResult wxDirTraverserMp3::OnDir(const wxString& WXUNUSED(dirname)) {
    if(stop || !rec)
        return wxDIR_STOP;
    else
        return wxDIR_CONTINUE;
}

/* ****************************************************************************************** */

void wxDirTraverserMp3::stopScan() {
    stop=true;
}


/* ******************************************************************************************
   ****************************************************************************************** */

ScanThread::ScanThread(MonArbo* a,wxString c,wxArrayString* f,bool rec):wxThread() {
    chemin=c;
    ma=a;
    file=f;
    recursive=rec;
    dtm=new wxDirTraverserMp3(file,recursive);
}

/* ****************************************************************************************** */

void *ScanThread::Entry() {
    int flags=wxDIR_FILES;
    if(recursive) flags |= wxDIR_DIRS;
    if(wxDirExists(chemin)) {
        wxDir d(chemin);
        wxLogNull ln;
        d.Traverse(*dtm);//get all mp3
    }
    return NULL;
}

/* ****************************************************************************************** */

void ScanThread::OnExit() {
    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, THREAD);
    event.SetInt(-1); // that's all
    wxPostEvent( ma, event );
}

/* ****************************************************************************************** */

void ScanThread::Stop() {
    dtm->stopScan();
}

/* ******************************************************************************************
   ****************************************************************************************** */
CopyMoveThread::CopyMoveThread(MaListe* l,MonBook* b,wxString& d,bool move) {
    ml = l;
    mb = b;
    mv = move;
    prog = (long)0;
    errors = 0;
    dossier = d;
    stop = false;
}

/* ****************************************************************************************** */

void CopyMoveThread::OnExit() {
    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_THREAD);
    event.SetInt(errors); // that's all
    wxPostEvent( mb, event );
}

/* ****************************************************************************************** */

void *CopyMoveThread::Entry() {
    long AncienneSel = ml->getSelection(-2);
    for (int h=0 ; h<ml->GetSelectedItemCount() ; h++) { //for each selected files
        if(stop) break;//if the treas is beein aborting
        wxString artiste = ml->getSelRowText(2,AncienneSel);
        wxString album = ml->getSelRowText(3,AncienneSel);
        wxString annee = ml->getSelRowText(4,AncienneSel);
        wxString genre = ml->getSelRowText(5,AncienneSel);
        wxString SaisieStr,path,path_ori;
        //Empty tags are remplaced whith explicit strings
        if (artiste == wxEmptyString)
            artiste = _("Unknown artist");
        if (album == wxEmptyString)
            album = _("Unknown album");
        if (annee == wxEmptyString)
            annee = _("Unknown year");
        if (genre == wxEmptyString)
            genre = _("Unknown genre");
        //clean '/'
        artiste.Replace(_T("/"),_T("_"));
        album.Replace(_T("/"),_T("_"));
        annee.Replace(_T("/"),_T("_"));
        genre.Replace(_T("/"),_T("_"));
        SaisieStr = mb->getComboOrg()->GetValue();
        //remplace the %?
        SaisieStr.Replace(_T("%1"), artiste.Trim());
        SaisieStr.Replace(_T("%2"), album.Trim());
        SaisieStr.Replace(_T("%3"), annee.Trim());
        SaisieStr.Replace(_T("%4"), genre.Trim());
        //clean string
        wxString rchar=OptionsHandler::getOption(_T("/notebook/replace_char"),wxString(_T("_")));
        if(rchar==wxEmptyString)
            rchar=_T("_");
        SaisieStr.Replace(_T("\""),rchar);
        mb->cleanStr(SaisieStr,rchar);


        path=dossier;
        wxStringTokenizer tkz(SaisieStr,wxT("/"));//split string on '/'
        try {
            wxRegEx backSlashes(wxT("\\\\\\*"));
            while(tkz.HasMoreTokens() ) {
                wxString token = tkz.GetNextToken();
                path.Printf(path+wxT("%c")+token,wxFileName::GetPathSeparator());
                backSlashes.Replace(&path,wxString(_T("\\")),0);
                //DirPortable::mkdirPortable((char const*)path.mb_str(wxConvISO8859_1));
                wxLogNull nolog;
                wxMkdir(path);
            }

            path.Printf(path+wxT("%c")+ml->getSelRowText(0,AncienneSel),wxFileName::GetPathSeparator());
            backSlashes.Replace(&path,wxString(_T("\\")),0);

            path_ori.Printf(ml->getSelRowText(8,AncienneSel)+wxT("%c")+ml->getSelRowText(0,AncienneSel),wxFileName::GetPathSeparator());
            prog = long(((h*1.0)/(ml->GetSelectedItemCount()*1.0))*100.0);
            if (h==ml->GetSelectedItemCount()-1) prog=(long)100;
            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_THREAD);
            event.SetInt(-1);
            event.SetString(wxString::Format(_T("%d"),prog));
            wxPostEvent( mb, event );

            if(mv) { //if files are moved
                if(!wxRenameFile(path_ori,path))
                    throw 1;//failed
                if(OptionsHandler::getOption(_T("/notebook/clean_dir"),true)==true)
                    wxRmdir(ml->getSelRowText(8,AncienneSel));//delete only empty folders
            } else { //if files are copyed
                if(!wxCopyFile(path_ori,path))
                    throw 1;//failed
            }
        } catch(...) {
            long index = ml->FindItem(-1,ml->getSelRowText(0,AncienneSel));
            ml->SetItemBackgroundColour(index,wxColour(220, 170, 170));//Set the item background color to red
            errors ++;

        }
        AncienneSel = ml->getSelection(AncienneSel);
    }

    return NULL;
}

/* ****************************************************************************************** */

long CopyMoveThread::getProg() {
    return prog;
}

/* ****************************************************************************************** */

int CopyMoveThread::getErrors() {
    return errors;
}

/* ****************************************************************************************** */

void CopyMoveThread::end() {
    stop=true;
}


/* ******************************************************************************************
   ****************************************************************************************** */
CopyDropThread::CopyDropThread(MonArbo* a,wxString data,wxString pt) {
    ma = a;
    files = data;
    path = pt;
    stop = false;
    prog = (long)0;
    errors = 0;
}

/* ****************************************************************************************** */

void CopyDropThread::OnExit() {
    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_THREAD);
    event.SetString(_T("End")); // that's all
    event.SetInt(errors);
    wxPostEvent( ma, event );
}

/* ****************************************************************************************** */

void *CopyDropThread::Entry() {
    wxStringTokenizer tkz(files,wxT("\n"));
    wxString pathNew;
    int i=0,nbt=tkz.CountTokens();
    //split files on '\n' to get each file
    while ( tkz.HasMoreTokens() ) {
        wxString token = tkz.GetNextToken();
        if(stop) break;
        i++;
        wxFileName fn(token);

        try {

            pathNew = path + wxFileName::GetPathSeparator() + fn.GetFullName();

            if(nbt==0)
                prog = (long)100;
            else
                prog = long(((i*1.0)/(nbt*1.0))*100.0);

            //send an event to MonArbo to update progbar, thread can't deal directly with GUI stuff
            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_THREAD);
            event.SetString(_T("up"));
            event.SetInt(prog);
            wxPostEvent( ma, event );

            wxLogNull ln;
            if(!wxCopyFile(token,pathNew)) {
                throw 1;
            }
        } catch(...) {
            errors ++;

        }
    }

    return NULL;
}

/* ****************************************************************************************** */

void CopyDropThread::end() {
    stop=true;
}

/* ****************************************************************************************** */

long CopyDropThread::getProg() {
    return prog;
}

/* ****************************************************************************************** *
 * Thread for checking new version of MO
 * ****************************************************************************************** */
UpdateThread::UpdateThread():wxThread() {

}

void *UpdateThread::Entry() {

    wxHTTP* get,*get2;
    bool isProxy=OptionsHandler::getOption(_T("/proxy/use_proxy"),0);
    bool isAut=OptionsHandler::getOption(_T("/proxy/use_login_proxy"),0);
    wxString pUser=OptionsHandler::getOption(_T("/proxy/login"));
    wxString pPwd=OptionsHandler::getOption(_T("/proxy/password"));
    wxString pUrl=OptionsHandler::getOption(_T("/proxy/address"));
    wxString pPort=OptionsHandler::getOption(_T("/proxy/port"));

    wxIPV4address addr;
    if(isProxy) {
        addr.Hostname(pUrl);
        addr.Service(pPort);
    }
    wxHTTP proxy;

    if(!isProxy || !proxy.Connect(addr,TRUE)) {
        get = new wxHTTP();
        get2  = get;
    } else {
        get = &proxy;
        get2 = new wxHTTP();
    }
    if(isProxy && isAut) {
        wxString strAut=pUser+_T(":")+pPwd;
        wxString strAutEnc=wxString((const char *)  Base64Handler::base64_encode((const char *)strAut.mb_str(wxConvUTF8),strAut.size()).c_str(),wxConvUTF8);
        get->SetProxyMode(true);
        get->SetHeader(_T("Proxy-Authorization"),wxT("Basic ") + strAutEnc);
    }


    //http://wiki.wxwidgets.org/WxHTTP
    get->SetHeader(_T("Content-type"), _T("text/html; charset=utf-8"));
    get2->SetHeader(_T("Content-type"), _T("text/html; charset=utf-8"));

    get2->SetTimeout(10); // 10 seconds of timeout instead of 10 minutes ...
    get->SetTimeout(10); // 10 seconds of timeout instead of 10 minutes ...

    if (get2->Connect(_T("www.musicorganizer.org"))) {
        wxInputStream *in_stream;
        if(isProxy) {
            in_stream = get->GetInputStream(_T("http://www.musicorganizer.org/version.php"));
        } else {
            in_stream = get2->GetInputStream(_T("/version.php"));
        }
        wxTextInputStream in_textstream(*in_stream, _T("\n"));

        if (in_stream != NULL) {
            wxString sVersion = in_textstream.ReadLine();
            wxString sVersionWithoutDot = sVersion;
            delete in_stream;
            while(sVersionWithoutDot.Find(wxChar('.')) != wxNOT_FOUND) {
                sVersionWithoutDot.Remove(sVersion.Find(wxChar('.')),1);
            }//Version without dot

            long iVersion;
            sVersionWithoutDot.ToLong(&iVersion);
            if (iVersion > IVERSION) { //New version is available
                wxString sFirstPart = _("Version ");
                wxString sSecondPart = _(" is available.");
                wxString sThirdPart = _("Would you like to download it now ?");
                wxMessageDialog msg(NULL,sFirstPart + sVersion + sSecondPart + _T("\n") + sThirdPart, _("A new version is available"),wxICON_QUESTION|wxYES_NO);
                if (msg.ShowModal() == wxID_YES) {
                    wxString sDownload = _T("http://www.musicorganizer.org/telechargements.php");
#ifdef __WXMAC__
                    sDownload = _T("http://download.tuxfamily.org/morg/files/MusicOrganizer-") + sVersion + _T(".dmg.bz2");
#elif __WXMSW__
                    sDownload = _T("http://download.tuxfamily.org/morg/files/Music.Organizer.Setup-") + sVersion + _T(".exe");
#endif
                    wxLaunchDefaultBrowser(sDownload);
                }
            }
        }
    }
    return NULL;
}

/* ****************************************************************************************** */

void UpdateThread::OnExit() {
}

/* ****************************************************************************************** */

void UpdateThread::Stop() {
}
